@@ -64,13 +64,19 @@ def random_individual(
     return current_genotype
 
 
-def mutate(r: Source, g: Grammar, ind: Genotype, max_depth: int) -> Genotype:
-    rkey = r.choice(
+def standard_mutate(r: Source, g: Grammar, ind: Genotype, max_depth: int, mutation_method) -> Genotype:
+    if mutation_method == 'all_codons_equal_prob':
+        weight = lambda key: len(ind.dna[key])
+    else:
+        weight = lambda key: 1
+    
+    rkey = r.choice_weighted(
         list(
             key
             for key in ind.dna.keys()
-            if (key != LEFTOVER_KEY)
+            if (len(ind.dna[key]) > 0)
         ),
+        list(weight(key) for key in ind.dna.keys() if (len(ind.dna[key]) > 0))
     )
     dna = ind.dna
     clone = [i for i in dna[rkey]]
@@ -79,6 +85,21 @@ def mutate(r: Source, g: Grammar, ind: Genotype, max_depth: int) -> Genotype:
     dna[rkey] = clone
     return Genotype(dna)
 
+def per_codon_mutate(r: Source, g: Grammar, ind: Genotype, max_depth: int, codon_prob) -> Genotype:
+    dna = ind.dna
+    for key in dna.keys():
+        for i in range(len(dna[key])):
+            if r.random_float(0,1) < codon_prob:
+                dna[key][i] = r.randint(0,MAX_RAND_INT)
+    return Genotype(dna)
+    
+def mutate(r: Source, g: Grammar, ind: Genotype, max_depth: int, mutation_method, codon_prob) -> Genotype:
+    if mutation_method == 'per_codon_mutate':
+        return per_codon_mutate(r, g, ind, max_depth, codon_prob)
+    else:
+        return standard_mutate(r, g, ind, max_depth, mutation_method)
+        
+
 
 def crossover(
     r: Source,
@@ -146,12 +167,20 @@ def create_tree(g: Grammar, ind: Genotype, depth: int, method) -> TreeNode:
 
 class StructuredGrammaticalEvolutionRepresentation(Representation[Genotype]):
     """This version uses a list of lists of integers to represent individuals, based on non-terminal
-    symbols."""
+    symbols.
+    
+    You can specify the [mutation_method] as follows:
+    - One mutation with all codons equal probabilities: all_codons_equal_prob
+    - One mutation with all genes equal probabilities: all_genes_equal_prob
+    - Mutation possibility for each codon (this also allows you to specify the [codon_prob]): per_codon_mutate
+    """
 
-    def __init__(self, depth = None, gene_size=256, method: Initialization_Method = PI_Grow()) -> None:
+    def __init__(self, depth = None, gene_size=256, method: Initialization_Method = PI_Grow(), mutation_method = 'all_codons_equal_prob', codon_prob = 0.05) -> None:
         self.depth = depth
         self.gene_size = gene_size
         self.method = method
+        self.mutation_method = mutation_method
+        self.codon_prob = codon_prob
 
     def create_individual(self, r: Source, g: Grammar, depth: int) -> Genotype:
         self.depth = depth
@@ -167,7 +196,8 @@ class StructuredGrammaticalEvolutionRepresentation(Representation[Genotype]):
         specific_type: type | None = None,
         depth_aware_mut: bool = False,
     ) -> Genotype:
-        return mutate(r, g, deepcopy(ind), depth)
+        new_ind = mutate(r, g, deepcopy(ind), depth, self.mutation_method, self.codon_prob)
+        return new_ind
 
     def crossover_individuals(
         self,
